@@ -2,6 +2,7 @@ import typing
 import sys
 import glob
 import os.path
+import traceback
 
 from . import loader
 from .jwt import JWT
@@ -14,6 +15,8 @@ from .openapi import Validator
 from .openapi import Operator
 from .openapi import Specification
 from .openapi import default_operation_id
+from .openapi import ScopesFetcher
+from .openapi import schematize
 from .exception import HTTPException
 from .exception import InternalServerError
 
@@ -23,14 +26,14 @@ PAYLOAD_OPERATIONS = {'post', 'put', 'options'}
 
 
 def respond_abort(request, exception: HTTPException):
-    return JSONResponse(**exception)
+    return request.app.response_class(**exception)
 
 
 def respond_capture(request, exception):
     return respond_abort(request, InternalServerError())
 
 
-class Fermata(object):
+class Fermata(ScopesFetcher):
 
     def __init__(
             self,
@@ -40,11 +43,16 @@ class Fermata(object):
             exception_captors: typing.Dict[typing.Type[Exception], typing.Callable] = None,
             scopes_fetcher: typing.Callable = None,
             jwt_key: str = None,
+            default_schematize = None,
+            response_class = None,
         ) -> None:
 
         self.default_package = default_package
-        self.scopes_fetcher = scopes_fetcher
+        if scopes_fetcher:
+            self.fetch_scopes = scopes_fetcher
         self.listeners = {}
+        self.default_schematize = default_schematize
+        self.response_class = response_class or JSONResponse
         if jwt_key:
             self.jwt = JWT(jwt_key)
         self.max_content_length = MAX_CONTENT_LENGTH
@@ -59,7 +67,6 @@ class Fermata(object):
             Exception: respond_capture,
             **(exception_captors or {})
         }
-        
 
     @property
     def name(self):
@@ -82,7 +89,7 @@ class Fermata(object):
         for path in glob.glob(spec_glob):
             ext = os.path.splitext(path)[1]
             if ext not in parsers:
-                self.trigger('log', 'warning', f'not supported file: {path}')
+                self.log('warning', f'not supported file: {path}')
                 continue
             spec_files.setdefault(parsers[ext], []).append(path)
         for parser, files in spec_files.items():
@@ -113,7 +120,7 @@ class Fermata(object):
                 if not security:
                     continue
                 path, operation = k.values()
-                authorizer = Authorizer(security, self.scopes_fetcher)
+                authorizer = Authorizer(security, self)
                 self.router.set(authorizer, base_path + path, operation)
 
         # validator
@@ -139,10 +146,14 @@ class Fermata(object):
             path, operation = k.values()
             if operation not in ALLOWED_OPERATIONS:
                 continue
-            path = base_path + path
-            op = node.get('operationId') or default_operation_id(path, operation)
-            operator = Operator(op, self.default_package)
-            self.router.set(operator, path, operation)
+            full_path = base_path + path
+            op = node.get('operationId') or default_operation_id(full_path, operation)
+            response_schema = spec.ref(
+                f'paths/{path.replace("/", "~1")}/{operation}/'
+                f'responses/200/content/'
+                f'{self.response_class.content_type.replace("/", "~1")}/schema')
+            operator = Operator(op, self.default_package, response_schema)
+            self.router.set(operator, full_path, operation)
 
         return self
 
@@ -152,7 +163,21 @@ class Fermata(object):
         params = validator.validate(request) if validator else {}
         res = operator.operate(request, **params)
         if not isinstance(res, Response):
-            res = JSONResponse(res)
+            if isinstance(res, tuple):
+                try:
+                    if isinstance(res[1], list):
+                        res = res[0], 200, res[1]
+                except IndexError:
+                    raise TypeError(
+                        "operator return tuple form must be one of: "
+                        "(body, status, headers), (body, status) or "
+                        "(body, headers).")
+                res = list(res)
+            else:
+                res = [res]
+            if operator.schema:
+                res[0] = schematize(operator.schema, res[0], self.default_schematize)
+            res = self.response_class(*res[:3])
         res.headers += headers
         return res
 
@@ -165,6 +190,13 @@ class Fermata(object):
                 return self.exception_captors[cls](request, exception)
         return respond_capture(request, exception)
 
+    def identity(self, request):
+        parts = request.headers.get('authorization', '').split(maxsplit=1)
+        if not parts or not self.jwt:
+            return None
+        token = self.jwt.loads(parts.pop())
+        return token
+
     def on(self, event, listener):
         self.listeners.setdefault(event, []).append(listener)
 
@@ -179,6 +211,9 @@ class Fermata(object):
             for l in self.listeners[event]:
                 l(*args, **kwargs)
 
+    def log(self, *args, **kwargs):
+        self.trigger('log', *args, **kwargs)
+
     def wsgi(
             self,
             environ: dict,
@@ -194,6 +229,7 @@ class Fermata(object):
             response = self.abort(request, e)
             self.trigger('abort', request=request, response=response)
         except Exception as e:
+            traceback.print_exc()
             response = self.capture(request, e)
             self.trigger('capture', request=request, response=response)
 
