@@ -19,16 +19,14 @@ def default_operation_id(path_exception, operation):
 
 class Operator:
 
-    def __init__(self, operation_id, default_package=None):
+    def __init__(self, operation_id, default_package=None, schema=None):
         if default_package and operation_id.startswith('.'):
             operation_id = default_package + operation_id
         self.operation_id = operation_id
+        self.schema = schema
 
     def load(self):
-        try:
-            self.operate = loader.load(self.operation_id)
-        except (ValueError, AttributeError, KeyError):
-            raise BadOperationId(f'Operation Not Found: `{self.operation_id}`')
+        self.operate = loader.load(self.operation_id)
         return self
 
     def operate(self, request, **kwargs):
